-------------------------------------------------------------------------------
--! @file       Inv_LBox_1.vhd
--! @author     William Diehl
--! @brief      SCREAM Inv_LBox_1
--! @date       29 Jun 2016
-------------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.numeric_std.all;

ENTITY Inv_LBox_1 IS

	PORT(
		Addr: IN STD_LOGIC_VECTOR(7 DOWNTO 0);
		Dout: OUT STD_LOGIC_VECTOR(15 DOWNTO 0)
	);

END Inv_LBox_1;

ARCHITECTURE dataflow OF Inv_LBox_1 IS

SIGNAL index: INTEGER RANGE 0 TO 255;
TYPE vector_array IS ARRAY (0 to 255) OF STD_LOGIC_VECTOR(15 DOWNTO 0);
CONSTANT memory : vector_array := 
	(

	x"0000", x"e79e", x"7704", x"909a", x"2ad1", x"cd4f", x"5dd5", x"ba4b",
	x"6313", x"848d", x"1417", x"f389", x"49c2", x"ae5c", x"3ec6", x"d958",
	x"dc05", x"3b9b", x"ab01", x"4c9f", x"f6d4", x"114a", x"81d0", x"664e",
	x"bf16", x"5888", x"c812", x"2f8c", x"95c7", x"7259", x"e2c3", x"055d",
	x"aef6", x"4968", x"d9f2", x"3e6c", x"8427", x"63b9", x"f323", x"14bd",
	x"cde5", x"2a7b", x"bae1", x"5d7f", x"e734", x"00aa", x"9030", x"77ae",
	x"72f3", x"956d", x"05f7", x"e269", x"5822", x"bfbc", x"2f26", x"c8b8",
	x"11e0", x"f67e", x"66e4", x"817a", x"3b31", x"dcaf", x"4c35", x"abab",
	x"2939", x"cea7", x"5e3d", x"b9a3", x"03e8", x"e476", x"74ec", x"9372",
	x"4a2a", x"adb4", x"3d2e", x"dab0", x"60fb", x"8765", x"17ff", x"f061",
	x"f53c", x"12a2", x"8238", x"65a6", x"dfed", x"3873", x"a8e9", x"4f77",
	x"962f", x"71b1", x"e12b", x"06b5", x"bcfe", x"5b60", x"cbfa", x"2c64",
	x"87cf", x"6051", x"f0cb", x"1755", x"ad1e", x"4a80", x"da1a", x"3d84",
	x"e4dc", x"0342", x"93d8", x"7446", x"ce0d", x"2993", x"b909", x"5e97",
	x"5bca", x"bc54", x"2cce", x"cb50", x"711b", x"9685", x"061f", x"e181",
	x"38d9", x"df47", x"4fdd", x"a843", x"1208", x"f596", x"650c", x"8292",
	x"82ae", x"6530", x"f5aa", x"1234", x"a87f", x"4fe1", x"df7b", x"38e5",
	x"e1bd", x"0623", x"96b9", x"7127", x"cb6c", x"2cf2", x"bc68", x"5bf6",
	x"5eab", x"b935", x"29af", x"ce31", x"747a", x"93e4", x"037e", x"e4e0",
	x"3db8", x"da26", x"4abc", x"ad22", x"1769", x"f0f7", x"606d", x"87f3",
	x"2c58", x"cbc6", x"5b5c", x"bcc2", x"0689", x"e117", x"718d", x"9613",
	x"4f4b", x"a8d5", x"384f", x"dfd1", x"659a", x"8204", x"129e", x"f500",
	x"f05d", x"17c3", x"8759", x"60c7", x"da8c", x"3d12", x"ad88", x"4a16",
	x"934e", x"74d0", x"e44a", x"03d4", x"b99f", x"5e01", x"ce9b", x"2905",
	x"ab97", x"4c09", x"dc93", x"3b0d", x"8146", x"66d8", x"f642", x"11dc",
	x"c884", x"2f1a", x"bf80", x"581e", x"e255", x"05cb", x"9551", x"72cf",
	x"7792", x"900c", x"0096", x"e708", x"5d43", x"badd", x"2a47", x"cdd9",
	x"1481", x"f31f", x"6385", x"841b", x"3e50", x"d9ce", x"4954", x"aeca",
	x"0561", x"e2ff", x"7265", x"95fb", x"2fb0", x"c82e", x"58b4", x"bf2a",
	x"6672", x"81ec", x"1176", x"f6e8", x"4ca3", x"ab3d", x"3ba7", x"dc39",
	x"d964", x"3efa", x"ae60", x"49fe", x"f3b5", x"142b", x"84b1", x"632f",
	x"ba77", x"5de9", x"cd73", x"2aed", x"90a6", x"7738", x"e7a2", x"003c"	

);

BEGIN

	index <= to_integer(unsigned(Addr));
	Dout <= memory(index);

END dataflow;