-------------------------------------------------------------------------------
--! @file       Tweak_Input.vhd
--! @author     William Diehl
--! @brief      SCREAM Tweak_Input
--! @date       29 Jun 2016
-------------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_unsigned.all;
USE ieee.numeric_std.all;

ENTITY Tweak_Input IS

	GENERIC(
	G_DBLK_SIZE: integer := 128;
	G_LBS_BYTES: integer := 4;
	G_MAX_LEN: integer := 32;
	NS: INTEGER:=12 -- Number of steps

	);
	
	PORT(

	 decrypt: in std_logic;
	 bdi_type : in std_logic_vector(3 - 1 downto 0);
	 tag_mode : in std_logic;
	 null_length : in std_logic;
	 bdi_size : in std_logic_vector(G_LBS_BYTES+1    -1 downto 0);
	 bdi_eot : in std_logic;
	 CN : std_logic_vector(G_MAX_LEN - 4 - 1 downto 0); 

-- CN is block counter, max number of blocks is G_MAX_LEN - 4

	 N: IN STD_LOGIC_VECTOR(87 DOWNTO 0);
-- Nonce (= IV) 11 bytes = 88 bits

	 T: OUT STD_LOGIC_VECTOR(G_DBLK_SIZE - 1 DOWNTO 0)
-- Tweak Input to Tweak Key scheduler
);

END Tweak_Input;

ARCHITECTURE dataflow OF Tweak_Input IS
constant zeros : std_logic_vector(36 - G_MAX_LEN - 1 downto 0):= (OTHERS=> '0');

SIGNAL  S0: STD_LOGIC_VECTOR(39 DOWNTO 0);
SIGNAL  S1 : STD_LOGIC_VECTOR(2 DOWNTO 0);
SIGNAL  S2 : STD_LOGIC_VECTOR(39 DOWNTO 0);
SIGNAL S3 : STD_LOGIC_VECTOR(39 DOWNTO 0);
SIGNAL T0, T1, T2, T_Dk : std_logic_vector(127 downto 0);
SIGNAL Txor : std_logic_vector(63 downto 0);
SIGNAL Nsteps: std_logic_vector(3 downto 0);
SIGNAL T_Dk_sel : std_logic_vector(2 downto 0);
SIGNAL Stag : std_logic_vector(3 downto 0);

BEGIN

  Stag <= x"0" when (null_length = '1') else bdi_size(G_LBS_BYTES - 1 downto 0);
  S0 <=  zeros & CN & Stag & "0001";  
  S1 <= "000" when (bdi_type(1) = '1') else
        "010" when (bdi_eot = '0') else
	     "100" when (bdi_size(G_LBS_BYTES) = '1') else
	     "110";
  
  S2 <= zeros & CN & "00000" & S1;

  S3 <= S0 when (tag_mode = '1') else S2;

  Nsteps <= std_logic_vector(to_unsigned(NS,4));
  T0 <= N & S3; 
  Txor <= T0(127 downto 64) xor T0(63 downto 0);
  T1 <= Txor & T0(127 downto 64);
  T2 <= T0(63 downto 0) & Txor;
  T_Dk_sel <= Nsteps(2 downto 0);
  
  with T_Dk_sel select
		T_Dk <= T2 when "000",
			     T0 when "001",
		        T1 when "010",
		        T2 when "011",
				  T0 when "100",
				  T0 when others;
		
 T <= T0 when (decrypt = '0' or (decrypt = '1' and bdi_eot = '1' and bdi_size(G_LBS_BYTES) = '0')) else T_Dk;

END dataflow; 	

